/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2021 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_serial.c
* Version      : CodeGenerator for RL78/G13 V2.05.06.02 [08 Nov 2021]
* Device(s)    : R5F100LE
* Tool-Chain   : CCRL
* Description  : This file implements device driver for Serial module.
* Creation Date: 2022/08/29
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_serial.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
volatile uint8_t   g_iic11_master_status_flag; /* iic11 start flag for send address check by master mode */
volatile uint8_t * gp_iic11_tx_address;        /* iic11 send data pointer by master mode */
volatile uint16_t  g_iic11_tx_count;           /* iic11 send data size by master mode */
volatile uint8_t * gp_iic11_rx_address;        /* iic11 receive data pointer by master mode */
volatile uint16_t  g_iic11_rx_count;           /* iic11 receive data size by master mode */
volatile uint16_t  g_iic11_rx_length;          /* iic11 receive data length by master mode */
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_SAU0_Create
* Description  : This function initializes the SAU0 module.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_SAU0_Create(void)
{
    SAU0EN = 1U;    /* supply SAU0 clock */
    NOP();
    NOP();
    NOP();
    NOP();
    SPS0 = _0000_SAU_CK00_FCLK_0 | _0000_SAU_CK01_FCLK_0;
    R_IIC11_Create();
}

/***********************************************************************************************************************
* Function Name: R_IIC11_Create
* Description  : This function initializes the IIC11 module.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_IIC11_Create(void)
{
    ST0 |= _0008_SAU_CH3_STOP_TRG_ON;    /* disable IIC11 */
    IICMK11 = 1U;    /* disable INTIIC11 interrupt */
    IICIF11 = 0U;    /* clear INTIIC11 interrupt flag */
    /* Set INTIIC11 high priority */
    IICPR111 = 0U;
    IICPR011 = 0U;
    SIR03 = _0004_SAU_SIRMN_FECTMN | _0002_SAU_SIRMN_PECTMN | _0001_SAU_SIRMN_OVCTMN;    /* clear error flag */
    SMR03 = _0020_SAU_SMRMN_INITIALVALUE | _0000_SAU_CLOCK_SELECT_CK00 | _0000_SAU_CLOCK_MODE_CKS |
            _0000_SAU_TRIGGER_SOFTWARE | _0000_SAU_EDGE_FALL | _0004_SAU_MODE_IIC | _0000_SAU_TRANSFER_END;
    SCR03 = _0000_SAU_TIMING_1 | _0000_SAU_INTSRE_MASK | _0000_SAU_PARITY_NONE | _0000_SAU_MSB | _0010_SAU_STOP_1 |
            _0007_SAU_LENGTH_8;
    SDR03 = _5200_IIC11_DIVISOR;
    SO0 |= _0800_SAU_CH3_CLOCK_OUTPUT_1 | _0008_SAU_CH3_DATA_OUTPUT_1;
    /* Set SCL11 pin */
    P3 |= 0x01U;
    PM3 &= 0xFEU;
    /* Set SDA11 pin */
    P5 |= 0x01U;
    PM5 &= 0xFEU;
}

/***********************************************************************************************************************
* Function Name: R_IIC11_Master_Send
* Description  : This function starts transferring data for IIC11 in master mode.
* Arguments    : adr -
*                    set address for select slave
*                tx_buf -
*                    transfer buffer pointer
*                tx_num -
*                    buffer size
* Return Value : None
***********************************************************************************************************************/
void R_IIC11_Master_Send(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num)
{
    g_iic11_master_status_flag = _00_SAU_IIC_MASTER_FLAG_CLEAR;    /* clear IIC11 master status flag */
    adr &= 0xFEU;    /* send mode */
    g_iic11_master_status_flag = _01_SAU_IIC_SEND_FLAG;            /* set master status flag */
    SCR03 &= ~_C000_SAU_RECEPTION_TRANSMISSION;
    SCR03 |= _8000_SAU_TRANSMISSION;
    /* Set paramater */
    g_iic11_tx_count = tx_num;
    gp_iic11_tx_address = tx_buf;
    /* Start condition */
    R_IIC11_StartCondition();
    IICIF11 = 0U;   /* clear INTIIC11 interrupt flag */
    IICMK11 = 0U;   /* enable INTIIC11 */
    SIO11 = adr;
}

/***********************************************************************************************************************
* Function Name: R_IIC11_Master_Receive
* Description  : This function starts receiving data for IIC11 in master mode.
* Arguments    : adr -
*                    set address for select slave
*                rx_buf -
*                    receive buffer pointer
*                rx_num -
*                    buffer size
* Return Value : None
***********************************************************************************************************************/
void R_IIC11_Master_Receive(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num)
{
    g_iic11_master_status_flag = _00_SAU_IIC_MASTER_FLAG_CLEAR;    /* clear master status flag */
    adr |= 0x01U;    /* receive mode */
    g_iic11_master_status_flag = _02_SAU_IIC_RECEIVE_FLAG;         /* set master status flag */
    SCR03 &= ~_C000_SAU_RECEPTION_TRANSMISSION;
    SCR03 |= _8000_SAU_TRANSMISSION;
    /* Set parameter */
    g_iic11_rx_length = rx_num;
    g_iic11_rx_count = 0U;
    gp_iic11_rx_address = rx_buf;
    /* Start condition */
    R_IIC11_StartCondition();
    IICIF11 = 0U;   /* clear INTIIC11 interrupt flag */
    IICMK11 = 0U;   /* enable INTIIC11 */
    SIO11 = adr;
}

/***********************************************************************************************************************
* Function Name: R_IIC11_Stop
* Description  : This function stops the IIC11 operation.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_IIC11_Stop(void)
{
    /* Stop transfer */
    IICMK11 = 1U;    /* disable INTIIC11 */
    ST0 |= _0008_SAU_CH3_STOP_TRG_ON;    /* disable IIC11 */
    IICIF11 = 0U;    /* clear INTIIC11 interrupt flag */
}

/***********************************************************************************************************************
* Function Name: R_IIC11_StartCondition
* Description  : This function starts IIC11 condition.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_IIC11_StartCondition(void)
{
    volatile uint8_t w_count;
    
    SO0 &= ~_0008_SAU_CH3_DATA_OUTPUT_1;    /* clear IIC11 SDA */
    
    /* Set delay to secure a hold time after SDA output low. The delay time depend on slave device.
       Here set 5us as default base on current clock */
    for (w_count = 0U; w_count <= IIC11_WAITTIME; w_count++)
    {
        NOP();
    }
    
    SO0 &= ~_0800_SAU_CH3_CLOCK_OUTPUT_1;    /* clear IIC11 SCL */
    SOE0 |= _0008_SAU_CH3_OUTPUT_ENABLE;            /* enable IIC11 output */
    SS0 |= _0008_SAU_CH3_START_TRG_ON;              /* enable IIC11 */

    /* Set delay to secure a hold time after SCL output low. The delay time depend on slave device.
       Here set 5us as default base on current clock */
    for (w_count = 0U; w_count <= IIC11_WAITTIME; w_count++)
    {
        NOP();
    }
}

/***********************************************************************************************************************
* Function Name: R_IIC11_StopCondition
* Description  : This function stops IIC11 condition.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_IIC11_StopCondition(void)
{
    volatile uint8_t w_count;
    
    ST0 |= _0008_SAU_CH3_STOP_TRG_ON;           /* disable IIC11 */
    SOE0 &= ~_0008_SAU_CH3_OUTPUT_ENABLE;       /* disable IIC11 output */
    SO0 &= ~_0008_SAU_CH3_DATA_OUTPUT_1;        /* clear IIC11 SDA */

    /* Set delay to secure a hold time after SDA output low. The delay time depend on slave device.
       Here set 5us as default base on current clock */
    for (w_count = 0U; w_count <= IIC11_WAITTIME; w_count++)
    {
        NOP();
    }

    SO0 |= _0800_SAU_CH3_CLOCK_OUTPUT_1; /* set IIC11 SCL */
    
    /* Set delay to secure a hold time after SCL output high. The delay time depend on slave device.
       Here set 5us as default base on current clock */
    for (w_count = 0U; w_count <= IIC11_WAITTIME; w_count++)
    {
        NOP();
    }
    
    SO0 |= _0008_SAU_CH3_DATA_OUTPUT_1;         /* set IIC11 SDA */

    /* Set delay to secure a hold time after SDA output high. The delay time depend on slave device.
       Here set 5us as default base on current clock */
    for (w_count = 0U; w_count <= IIC11_WAITTIME; w_count++)
    {
        NOP();
    }
}

/* Start user code for adding. Do not edit comment generated here */

/***********************************************************************
* Function Name: R_IIC11_Master_Receive2
* Description  : ̊֐͒ʐMXe[^XNAĎMN܂B
*               imubLO̎MN֐łBM
*               R_IIC11_Get_status֐ŊmFĂj
* Arguments    : adr -
*                    set address for select slave
*                rx_buf -
*                    receive buffer pointer
*                rx_num -
*                    buffer size
* Return Value : Ȃ
************************************************************************/
void R_IIC11_Master_Receive2(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num)
{

    R_IIC11_Clear_status();                     /* ʐMXe[^XʐM */

    R_IIC11_Master_Receive(adr,rx_buf, rx_num); /* MN   */

}   /* end of function R_IIC11_Master_Receive2  */


/***********************************************************************
* Function Name: R_IIC11_Master_Send2
* Description  : ̊֐͒ʐMXe[^XNAđMN܂B
*               imubLȎMN֐łBM
*               R_IIC11_Get_status֐ŊmFĂj
* Arguments    : adr -
*                    set address for select slave
*                tx_buf -
*                    transfer buffer pointer
*                tx_num -
*                    buffer size
* Return Value : Ȃ
************************************************************************/
void R_IIC11_Master_Send2(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num)
{

    R_IIC11_Clear_status();                     /* ʐMXe[^XʐM */

    R_IIC11_Master_Send(adr, tx_buf, tx_num);   /* MN */

}   /* end of function R_IIC11_Master_Send2  */

/***********************************************************************
* Function Name: R_IIC11_Master_Receive2
* Description  : ̊֐͒ʐMXe[^XNAĎMN܂B
*               imubLO̎MN֐łBM
*               R_IIC11_Get_status֐ŊmFĂj
* Arguments    : adr -
*                    set address for select slave
*                rx_buf -
*                    receive buffer pointer
*                rx_num -
*                    buffer size
* Return Value : ʐM
*               0x01: complete
*               0x81: finished by error
************************************************************************/
uint8_t R_IIC11_Master_Receiv3(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num)
{

    uint8_t work = 0x00;

    R_IIC11_Clear_status();                     /* ʐMXe[^XʐM */

    R_IIC11_Master_Receive(adr, rx_buf, rx_num); /* MN   */

    while ( 0 == work )
    {
        work = R_IIC11_Get_status();            /* ʐMXe[^Xǂݏo   */
    }
    
    return(work);                               /* ʐMʂ߂           */

}   /* end of function R_IIC11_Master_Receive2  */


/***********************************************************************
* Function Name: R_IIC11_Master_Send3
* Description  : ̊֐͒ʐMXe[^XNAđMNA
*               M҂Ă߂܂B
*               iM̃ubLO֐łj
* Arguments    : adr -
*                    set address for select slave
*                tx_buf -
*                    transfer buffer pointer
*                tx_num -
*                    buffer size
* Return Value : ʐM
*               0x01: complete
*               0x81: finished by error
************************************************************************/
uint8_t R_IIC11_Master_Send3(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num)
{

    uint8_t work = 0x00;

    R_IIC11_Clear_status();                     /* ʐMXe[^XʐM */

    R_IIC11_Master_Send(adr, tx_buf, tx_num);   /* MN */

    while ( 0 == work )
    {
        work = R_IIC11_Get_status();            /* ʐMXe[^Xǂݏo   */
    }
    
    return(work);                               /* ʐMʂ߂           */

}   /* end of function R_IIC11_Master_Send2  */

/* End user code. Do not edit comment generated here */
