/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2021 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_serial_user.c
* Version      : CodeGenerator for RL78/G13 V2.05.06.02 [08 Nov 2021]
* Device(s)    : R5F100LE
* Tool-Chain   : CCRL
* Description  : This file implements device driver for Serial module.
* Creation Date: 2022/08/29
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_serial.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
#pragma interrupt r_iic11_interrupt(vect=INTIIC11)
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
extern volatile uint8_t   g_iic11_master_status_flag;  /* iic11 start flag for send address check by master mode */
extern volatile uint8_t * gp_iic11_tx_address;         /* iic11 send data pointer by master mode */
extern volatile uint16_t  g_iic11_tx_count;            /* iic11 send data size by master mode */
extern volatile uint8_t * gp_iic11_rx_address;         /* iic11 receive data pointer by master mode */
extern volatile uint16_t  g_iic11_rx_count;            /* iic11 receive data size by master mode */
extern volatile uint16_t  g_iic11_rx_length;           /* iic11 receive data length by master mode */
/* Start user code for global. Do not edit comment generated here */
static volatile uint8_t    g_iic11_status;             /* IIC11ʐMXe[^X
                    0x00    : ʐMsi҂j
                    0x01    : ʐMI
                    0x81    : ʐMG[
*/
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: r_iic11_interrupt
* Description  : This function is INTIIC11 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void __near r_iic11_interrupt(void)
{
    volatile uint16_t w_count;

    /* Set delay to start next transmission. The delay time depend on slave device.
       Here set 20us as default base on current clock */
    for (w_count = 0U; w_count <= IIC11_WAITTIME_2; w_count++)
    {
        NOP();
    }

    if (((SSR03 & _0002_SAU_PARITY_ERROR) == 0x0002U) && (g_iic11_tx_count != 0U))
    {
        SIR03 |= _0002_SAU_SIRMN_PECTMN; //clear ACK error detection flag
        R_IIC11_StopCondition();
        r_iic11_callback_master_error(MD_NACK);
    }
    else if(((SSR03 & _0001_SAU_OVERRUN_ERROR) == 0x0001U) && (g_iic11_tx_count != 0U))
    {
        SIR03 |= _0001_SAU_SIRMN_OVCTMN; //clear overrun error detection flag
        R_IIC11_StopCondition();
        r_iic11_callback_master_error(MD_OVERRUN);
    }
    else
    {
        /* Control for master send */
        if ((g_iic11_master_status_flag & _01_SAU_IIC_SEND_FLAG) == 1U)
        {
            if (g_iic11_tx_count > 0U)
            {
                SIO11 = *gp_iic11_tx_address;
                gp_iic11_tx_address++;
                g_iic11_tx_count--;
            }
            else
            {
                /* IIC master transmission finishes and a callback function can be called here. */
                r_iic11_callback_master_sendend();
            }
        }
        /* Control for master receive */
        else 
        {
            if ((g_iic11_master_status_flag & _04_SAU_IIC_SENDED_ADDRESS_FLAG) == 0U)
            {
                ST0 |= _0008_SAU_CH3_STOP_TRG_ON;
                SCR03 &= ~_C000_SAU_RECEPTION_TRANSMISSION;
                SCR03 |= _4000_SAU_RECEPTION;
                SS0 |= _0008_SAU_CH3_START_TRG_ON;
                g_iic11_master_status_flag |= _04_SAU_IIC_SENDED_ADDRESS_FLAG;
                
                if (g_iic11_rx_length == 1U)
                {
                    SOE0 &= ~_0008_SAU_CH3_OUTPUT_ENABLE;    /* disable IIC11 out */
                }
                
                SIO11 = 0xFFU;
            }
            else
            {
                if (g_iic11_rx_count < g_iic11_rx_length)
                {
                    *gp_iic11_rx_address = SIO11;
                    gp_iic11_rx_address++;
                    g_iic11_rx_count++;
                    
                    if (g_iic11_rx_count == (g_iic11_rx_length - 1U))
                    {
                        SOE0 &= ~_0008_SAU_CH3_OUTPUT_ENABLE;    /* disable IIC11 out */
                        SIO11 = 0xFFU;
                    }
                    else if (g_iic11_rx_count == g_iic11_rx_length)
                    {
                        /* IIC master reception finishes and a callback function can be called here. */
                        r_iic11_callback_master_receiveend();
                    }
                    else
                    {
                        SIO11 = 0xFFU;
                    }
                }
            }
        }
    }
}

/***********************************************************************************************************************
* Function Name: r_iic11_callback_master_error
* Description  : This function is a callback function when IIC11 master error occurs.
* Arguments    : flag -
*                    status flag
* Return Value : None
***********************************************************************************************************************/
static void r_iic11_callback_master_error(MD_STATUS flag)
{
    /* Start user code. Do not edit comment generated here */
    g_iic11_status = 0x81;              // ʐMXe[^XG[
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_iic11_callback_master_receiveend
* Description  : This function is a callback function when IIC11 finishes master reception.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_iic11_callback_master_receiveend(void)
{
    /* Start user code. Do not edit comment generated here */
    g_iic11_status = 0x01;              // ʐMI
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_iic11_callback_master_sendend
* Description  : This function is a callback function when IIC11 finishes master transmission.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_iic11_callback_master_sendend(void)
{
    /* Start user code. Do not edit comment generated here */
    g_iic11_status = 0x01;              // ʐMI
    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */

/***********************************************************************
* Function Name: R_IIC11_Clear_status
* Description  : ̊֐͒ʐM󋵃tONA܂B
* Arguments    : Ȃ
* Return Value : Ȃ
************************************************************************/
void R_IIC11_Clear_status(void)
{

    g_iic11_status = 0x00;                      /* ʐMtONA */

}   /* end of function R_IIC11_Clear_status   */

/***********************************************************************
* Function Name: R_IIC11_Get_status
* Description  : ̊֐͒ʐM󋵂߂܂B
* Arguments    : Ȃ
* Return Value : ʐMXe[^X
*               0x01: complete
*               0x81: finished by error
************************************************************************/
uint8_t R_IIC11_Get_status(void)
{

    return( g_iic11_status );                   /* ʐMXe[^X߂   */

}   /* end of function R_IIC11_Get_status   */


/***********************************************************************
* Function Name: R_IIC11_Wait_finish
* Description  : ̊֐͒ʐM҂܂B
* Arguments    : Ȃ
* Return Value : ʐM
*               0x01: complete
*               0x81: finished by error
************************************************************************/
uint8_t R_IIC11_Wait_finish(void)
{
    while ( 0 == g_iic11_status )
    {                                           /* ʐM҂     */
        NOP();
    }

    return( g_iic11_status );                   /* ʐMʂ߂   */

}   /* end of function R_IIC11_Wait_finish  */

/* End user code. Do not edit comment generated here */
